(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
	typeof define === 'function' && define.amd ? define(factory) :
	(global.setState = factory());
}(this, (function () { 'use strict';

var toString = Object.prototype.toString;

/**
 * 判断对象的类型
 * @param  {*}      obj  需要判断类型的对象
 * @param  {string|Array} [types=''] 对象的类型
 * @return {boolean}     入参对象是否与给定的对象类型匹配
 *
 * @example
 *
 * isType(null, 'null')
 * // => true
 *
 * isType({}, 'object')
 * // => true
 *
 * isType([], 'ARRAY')
 * // => true
 *
 * isType(undefined, 'undefined')
 * // => true
 *
 * isType([], ['string', 'array', 'BOOLEAN'])
 */

var isType = (function (obj) {
  var types = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';

  var type = toString.call(obj).slice(8, -1).toLowerCase();

  return Array.isArray(types) ? types.map(function (t) {
    return t.toLowerCase();
  }).includes(type) : types.toLowerCase() === type;
});

var isType_1 = isType;

function setState(Vue) {
  function assignmentProp2State(vm, state) {
    for (var prop in state) {
      vm[prop] = state[prop];
    }
  }

  function mountState2VM(state) {
    if (state == null) return;

    if (isType_1(state, 'Object')) {
      assignmentProp2State(this, state);
    }

    if (isType_1(state, 'Function')) {
      assignmentProp2State(this, state.call(this, this));
    }
  }

  Vue.prototype.setState = function (state) {
    mountState2VM.call(this, state);
    this.$nextTick(this.$forceUpdate);
  };

  Vue.mixin({
    beforeCreate: function () {
      mountState2VM.call(this, this.$options.state);
    }
  });
}

if (typeof window !== 'undefined' && window.Vue) {
  setState(window.Vue);
}

return setState;

})));
